#!/bin/sh -eu

. ../base.lib.sh
. ../string.lib.sh
. ../bool.lib.sh
. ../number.lib.sh

test_k_int_parse () {
	assertEquals "'10' is 10" 10 "$(k_int_parse "10")"
	assertEquals "'8' is 8" 8 "$(k_int_parse "8")"
	assertEquals "8 is 8" 8 "$(k_int_parse 8)"
	assertEquals "'10.0' is 10" 10 "$(k_int_parse "10.0")"
	assertEquals "'999999' is 999999" 999999 "$(k_int_parse "999999")"
	assertEquals "'0xFF' is 255" 255 "$(k_int_parse "0xFF")"
	assertEquals "'abc', '10' is 10" 10 "$(k_int_parse "abc" "10")"
	assertEquals "'def', '0xFF' is 255" 255 "$(k_int_parse "def" "0xFF")"
	assertEquals "'12.3', '11' is 12" 12 "$(k_int_parse "12.3" "11")"
	assertEquals "'12.abc', '11' is 11" 11 "$(k_int_parse "12.abc" "11")"
}

test_k_int_parse_error (){
	local output=''


	output="$(k_int_parse_error "10" 2>&1)"
	assertTrue "10 can be parsed as int" $?
	assertEquals '10 is 10' '10' "${output}"

	output="$(k_int_parse_error "8" 2>&1)"
	assertTrue "8 can be parsed as int" $?
	assertEquals '8 is 8' '8' "${output}"

	output="$(k_int_parse_error 8 2>&1)"
	assertTrue "8 can be parsed as int" $?
	assertEquals '8 is 8' 8 "${output}"

	output="$(k_int_parse_error "abc" 2>&1)"
	assertFalse "abc cannot be parsed as int" $?
	assertEquals 'abc is not int' 'Cannot parse "abc" as integer' "${output}"

	output="$(k_int_parse_error "10.0" 2>&1)"
	assertTrue "10.0 can be parsed as int" $?
	assertEquals '10.0 is 10' '10' "${output}"

	output="$(k_int_parse_error "10.5" 2>&1)"
	assertTrue "10.5 can be parsed as int" $?
	assertEquals '10.5 is 10' '10' "${output}"

	output="$(k_int_parse_error "10.abc" 2>&1)"
	assertFalse "10.abc cannot be parsed as int" $?
	assertEquals '10.abc is not int' 'Cannot parse "10.abc" as integer' "${output}"

	output="$(k_int_parse_error "999999" 2>&1)"
	assertTrue "999999 can be parsed as int" $?
	assertEquals '999999 is 999999' '999999' "${output}"

	output="$(k_int_parse_error "0xFF" 2>&1)"
	assertTrue "0xFF can be parsed as int" $?
	assertEquals '0xFF is 255' '255' "${output}"
}

